
/*--------------------------------------------------------------
 *
 * Name:        DAC.C
 *
 * Purpose:     Set D/A converter of signal generator.
 *
 * Inputs:      Command line.   See usage() for information.
 *
 * Outputs:     RS-232 port.
 *
 * Description: The circuit contains a quad D/A converter.
 *              This can set the output of each separately.
 *              See sg_cmd.h for more info.
 *
 * History:     Original, John Langner, WB2OSZ.
 *
 *--------------------------------------------------------------*/

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <bios.h>
#include <math.h>


#include "sg_cmd.h"


void usage();



int main (int argc, char *argv[])
{
        FILE *fp;

/*
 * From the command line.
 */
        int com_port_num;               /* For communicating with device. */
        enum dac_addr_e dac_address;
        int dac_value;
        char digits[20];

        int n;
        char port_name[10];

/*
 * Get the items from the command line.
 */
        com_port_num = 1;
        dac_address = DAC_A;

        n = 1;

        if (n >= argc) {
            usage();
        }

        if (strnicmp(argv[n], "/com=", 5) == 0) {
            com_port_num = atoi(argv[n]+5);
            printf ("Using COM port %d.\n", com_port_num);
            n++;
        }

        if (isalpha(argv[n][0])) {

            int x = argv[n][0];

            switch (x) {
                case 'a':
                case 'A':
                    dac_address = DAC_A;
                    break;

                case 'b':
                case 'B':
                    dac_address = DAC_B;
                    break;

                case 'c':
                case 'C':
                    dac_address = DAC_C;
                    break;

                case 'd':
                case 'D':
                    dac_address = DAC_D;
                    break;

                case 'x':                       /* Actually X = C */
                case 'X':
                    dac_address = DAC_X;
                    break;

                case 'z':                       /* Actually Z = D */
                case 'Z':
                    dac_address = DAC_Z;
                    break;

                default:
                    fprintf (stderr, "Invalid DAC address: %c\n", x);
                    usage ();
            }

            n++;
        }

        if (n >= argc) {
            fprintf (stderr, "DAC value is missing.\n\n");
            usage();
        }

        dac_value = atoi (argv[n]);
        n++;


        if (n < argc) {
            fprintf (stderr, "Too many command line arguments.\n\n");
            usage ();
        }

/*
 * Enforce some limits.
 */
        if (dac_value < 0)
            dac_value = 0;

        if (dac_value > 255)
            dac_value = 255;


/*
 * Send it out serial port.
 * This is a one way message; no ack comes back.
 *
 * _bios_serialcom is for Borland C.
 * Might not work for other compilers.
 */

        _bios_serialcom (_COM_INIT,
                com_port_num - 1,               /* 0 for COM1. */
                _COM_CHR8 | _COM_STOP1 | _COM_NOPARITY | _COM_300);

        sprintf (port_name, "COM%d", com_port_num);

        fp = fopen (port_name, "wb");
        if (fp == NULL) {
            fprintf (stderr, "ERROR: Couldn't open device %s.\n", port_name);
            exit (1);
        }

        /* printf ("DEBUG: %c %d %d \n", DAC_CMD, dac_address, dac_value); */
        
        fputc (DAC_CMD, fp);
        fputc (dac_address, fp);
        fputc (dac_value, fp);

        fclose (fp);

        return (0);     /* Quiet compiler warning. */
}



/*-----------------------------------------------------------------
 *
 * Name:        usage
 *
 * Purpose:     Print brief usage message and exit.
 *
 *----------------------------------------------------------------*/

void usage ()
{
        fprintf (stderr, "usage: \n");
        fprintf (stderr, "        DAC [ /COM=n ] [ addr ] \"xxx\"\n");
        fprintf (stderr, "where,\n");
        fprintf (stderr, "        n         is optional COM port number.  Default 1.\n");
        fprintf (stderr, "        addr      is optional channel: A, B, C (or X), D (or Z).\n");
        fprintf (stderr, "                      Default is A for amplitude.\n");
        fprintf (stderr, "        xxx       is value for DAC.\n");
        fprintf (stderr, "                      0 = maximum, 255 = minimum for A.\n");
        fprintf (stderr, "                      0 = 0 volts, 255 = 5 volts for others.\n");
        fprintf (stderr, "\n");

        exit (1);

} /* end usage */

/* end DAC.C */
