/*------------------------------------------------------------------------
 *
 * File:        DDS.C
 *
 * Purpose:     DDS Signal Generator using AD9850.
 *
 * Inputs:      Rotary encoder switch.
 *              Commands from RS-232 port.
 *
 * Outputs:     LCD for frequency display and messages.
 *              (Generic - 1 line x 16 characters, available
 *              from at least a half dozen manufacturers.
 *              All use HD44780 or SED1278 controller.)
 *
 *              Serial commands to AD9850 digital frequency
 *              synthesizer.
 *
 *              Commands to quad D/A converter for adjusting
 *              signal output level and two auxilliary voltage
 *              outputs.
 *
 * References:  "Building a Direct Digital Synthesis VFO"
 *              Curtis W. Preuss, WB2V, QEX, July 1997
 *
 *		    http://wwwn.njqrp.org/mbrproj/9850dds.html
 *
 *		AD9850:
 *
 *		    http://products.analog.com/products/
 *			info.asp?product=AD9850
 *
 *              MAX509:
 *
 *                  http://pdfserv.maxim-ic.com/arpdf/MAX509-MAX510.pdf
 *
 *		LCD:
 *
 *		    http://eio.com/lcdintro.htm	
 *		    http://www.eea.epson.com/pdfs/sed1200.pdf
 *
 *		PIC 16F84:
 *
 *		    http://www.microchip.com/download/lit/picmicro/
 *			families/16f84x/datasheet/30430c.pdf
 *
 *              Hi-Tech C Compiler:
 *
 *                  http://www.hitech.com.au
 *
 *                      The full commercial version is a bit pricey
 *                      for student/hobbyist use but they have a
 *                      free reduced functionality version for the
 *                      16C84 / 16F84 only.
 *
 *                      Why would anyone want to use assembly langugage???
 *
 *
 * Description: This is intended to be a more general purpose
 *              signal generator instead of a VFO for a radio.
 *              It's written in C, rather than assembly language
 *              so it is much easier to write, understand, and
 *              modify later.
 *
 * Revisions:   Release 1 - July 2001, John Langner, WB2OSZ
 *
 * Next Version: This little gadget has become a valuable addition to 
 *              the home electronics workbench.
 *
 *              The one big disappointment is the very slow sweep
 *              speed when using this to examine filter response.
 *              The major bottleneck is shifting all those bits out
 *              one by one to the DDS chip and D/A converter.
 *			
 *              The next revision (if there ever is one) will use
 *              the parallel interface to the DDS chip and a different
 *              DAC with a parallel interface.  Of course this will 
 *              require more I/O bits so we'll need something like
 *              the 16F87x instead of the good old 16F84.
 *
 *              This will offer other advantages such as more memory,
 *              hardware UART, and a higher clock speed.
 *
 *------------------------------------------------------------------------*/

#if 1		/* Original circuit had 4 MHz. */

#define XTAL    4000000         /* Xtal frequency.  Need for */
 				/* various timings. */

#else		/* Didn't have 4 MHz in my junk box, but I did have ... */

#define XTAL	2457600

#endif
 


#include	<pic.h>

/*------------------------------------------------------------------------
 *
 * Certain options must be selected when programming the PIC.
 * 
 * Select
 *      code protect off,
 *      power up timer on,
 *      watch dog timer off,
 *      XT oscillator.
 *
 * Compiler manual mentions symbols like FOSC1 but I don't see 
 * them anywhere.
 *
 * It should be possible for the compiler to put the required
 * options in the file for the programmer but I haven't been able
 * to figure this out yet.
 *
 *
 * The configuration word is different for the C and F versions:
 *
 * 16C84:
 *
 *      FOSC0   0x01            1 = XT osc.
 *      FOSC1   0x02            0 = XT osc.
 *      WDTE    0x04            0 = Watch dog timer off.
 *      PWRTE   0x08            1 = Power on timer enable. 
 *	CP	0x10		1 = Code Protection off. 
 *
 * 16F84:
 *
 *      FOSC0   0x0001          1 = XT osc.
 *      FOSC1   0x0002          0 = XT osc.
 *      WDTE    0x0004          0 = Watch dog timer off.
 *      PWRTE   0x0008          0 = Power on timer enable. (opposite)
 *	CP	0x3ff0		all 1 = Code Protection off. 
 *
 *-----------------------------------------------------------------------*/

/* 0b11001 produces "digit out of range" error. */
/* Use the assember version with trailing upper case B. */


#if _16C84
 
__CONFIG(11001B);                       

#error This application requires the 16F84 because */
#error the 16C84 does not have enough RAM. */

#elif _16F84

__CONFIG(0011111111110001B);

#else

#error Wrong device type.

#endif


#define TRUE    1
#define FALSE   0


#define NOP     asm("NOP")



/*
 * weight.h is generated by the accompanying weight.c program.
 *
 * It contains a table of constants for converting a frequency,
 * in BCD, to a binary number for the AD9850.
 *
 * These constants are a function of the clock frequency.
 * They are placed in the data EEPROM, rather than embedded
 * in the program, so there is the potential of loading a new
 * set of calibration data without erasing the program.
 *
 * sg_cmd.h defines:
 *
 *      DIGITS - number of digits in a BCD array, currently 9.
 *	DDS_PREC - number of bytes per conversion constant,
 *		currently 5.  
 *
 *      Constants and structures for commands thru serial port.
 *
 * The current code is not written in a general enough way that 
 * these can be changed without other modifications.
 */

#include "sg_cmd.h"

#include "weight.h"

#if DIGITS == 8
/* 1 Hz steps from 0 to 20,000,000 */
#elif DIGITS == 9
/* 0.1 Hz steps from 0.0 to 20,000,000.0 */
#else
#error  Unexpected DIGITS - expect trouble.
#endif

#if DDS_PREC != 5
#error   Unexpected DDS_PREC - expect trouble.
#endif
 					
/*
 * Functions appear in this order:
 *
 * Process inputs from rotary encoder and serial port.
 */

void twist_event (byte_t *freq, byte_t *cursor_digit);


/* Commands from computer. */

void cmd_frequency (byte_t *freq, byte_t cursor_digit);
void cmd_message ();
void cmd_dac ();
void cmd_sweep ();

 
/* BCD number manipulation. */

void bcd_clear (byte_t *bcd);
void bcd_incr (byte_t *bcd, byte_t position);
void bcd_decr (byte_t *bcd, byte_t position);
void bcd_display (byte_t *bcd, byte_t cursor_digit);


/* Five byte binary number arithmetic. */

void long5_add (byte_t *psum, byte_t *padd);
void long5_negate (byte_t *pnum);


/* Hardware specific. */

byte_t data_eeprom_read (byte_t adr);
void data_eeprom_write (byte_t adr, byte_t dat);
void delay (byte_t ms);


/* Liquid Crystal Display. */

void lcd_init (void);
void lcd_put4 (byte_t x);
byte_t lcd_read_bf_ac (void);
void lcd_write_data (byte_t x);
void lcd_write_instr (byte_t x);
void lcd_set_cursor (byte_t position);
void lcd_show_cursor (void);
void lcd_hide_cursor (void);


/* Digital frequency synthesizer. */

void dds_init (void);
void dds_from_bcd (byte_t *bcd);
void dds_write_freq (byte_t *f);
void dds_shift_byte (byte_t x);


/* Quad D/A converter. */

void dac_init ();
void dac_write (enum dac_addr_e addr, byte_t value);


/* Last, but not least, input functions for serial port */
/* and the rotary encoder. */

void interrupt timer (void);
byte_t ar_sample (void);
byte_t ar_ch_avail (void);
byte_t ar_get_ch (void);
void enc_sample (void);


/*------------------------------------------------------------------------
 *
 *                          I/O Map
 *                          =======
 *
 * Port Bit I/O Pin     to Where        Description
 * ---- --- --- ---     --------        -----------
 *
 *   A   0   i  17      Rotary Encoder  B
 *   A   1   i  18      Rotary Encoder  A
 *   A   2   i  1       Rotary Encoder  Push in
 *
 *   A   3   o  2       MAX509, pin 15	-CS, chip select
 *			Quad DAC	Original circuit used this to
 *			(see below)	power the LCD.  
 *					The LCD power is now connected
 *					directly to +5V and this is used
 *					to control a digital to analog
 *					converter.
 *
 *   A   4   i  3       RS-232 port     Original circuit used PIC16C54
 *			(see below)	and this was RTCC input tied to
 *					+5 volts.
 *					On the PIC16x84, this is a general
 *					purpose I/O line.  We are using it
 *					for a serial port input so a 
 *					computer  can control the 
 *					signal generator.
 *
 *   B   0   o  6       AD9850, pin 8   FQ_UD, frequency update 
 *
 *   B   1   o  7       LCD, pin 4      RS, Register Select
 *
 *   B   2   o  8       LCD, pin 5      R/-W
 *
 *   B   3   o  9       LCD, pin 6      E
 *
 *   B   4   io 10      LCD, pin 11     D4
 *			MAX509, pin 13	SCLK, serial clock
 *
 *   B   5   io 11      LCD, pin 12     D5
 *                      AD9850, pin 7   W_CLK, word load clock
 *
 *   B   6   io 12      LCD, pin 13     D6
 *			MAX509, pin 12	DIN, data in
 *
 *   B   7   io 13      LCD, pin 14     D7
 *                      AD9850, pin 25  D7, serial data
 *
 *
 *
 * OPTIONAL SERIAL PORT
 *
 *      We simply use a 14C89 to convert the bipolar RS-232 signal
 *	to CMOS levels.
 *
 *	DB9S connector	14C89	PIC	other / notes
 *	--------------	-----	---	-------------
 *	1	CD			
 *	2	RD			
 *	3	TD	13		Data from computer
 *			11	3
 *	4	DTR			
 *	5	SG	7		Ground
 *	6	DSR			1k resistor to 5 volts
 *	7	RTS
 *	8	CTS			1k resistor to 5 volts
 *	9	RI
 *			14		5 volts			
 *
 *	If the serial port is not used, attach PIC pin 3 to +5v
 *	as in the original circuit.
 *
 *
 * 
 * OPTIONAL QUAD D/A CONVERTER
 *
 *	A quad D/A converter is used to adjust the signal amplitude
 *	and feed two other anlog outputs which would generally go
 *	to an oscilloscope.
 *
 *	MAX509		PIC	other / notes
 *	-------------	---	-------------
 *	1	OUT B		not used
 *	2	OUT A		Disconnect ground end of 3.9k R4 
 *				and put it here instead so DAC can
 *				control the signal amplitude.
 *	3	VSS		Ground
 *	4	REF B		5 volts
 *	5	REF A		Trim pot between ground and 5 volts.
 *				Adjust to point where output from DDS
 *				just disapears when DAC OUT A is at maximum.
 *				Should be around 1.25 volts.
 *	6	AGND		Ground
 *	7	N.C.
 *	8	DGND		Ground
 *	9	-LDAC		5 volts
 *	10	N.C.
 *	11	-CLR		5 volts
 *	12	DIN	12	serial data in from PIC, port B, bit 6
 *	13	SCLK	10	serial clock from PIC, port B, bit 4
 *	14	N.C.
 *	15	-CS	2	chip select from PIC, port A, bit 3
 *	16	REF D		5 volts
 *	17	REF C		5 volts
 *	18	VDD		5 volts
 *	19	OUT D		Marker "Z" output connector.
 *	20	OUT C		Sweep "X" output connector.	
 *	
 *
 *
 *		* * * *   OTHER PCB NOTES   * * * *
 *
 *
 * There is a definite mistake and a mystery in the PC board available 
 * from FAR Circuits.  These involve the 14 connection points which
 * which go to the LCD.
 *
 *	pin 1 - open, should be ground.  ** important **
 *
 *	pins 7,9 - connected to ground.  Should be disconnected.
 *	pins 8,10 - connected to pin 6.  Should be disconnected.
 *
 *		These are unused data lines when using four bit 
 *		data mode.  They have internal pull ups and can 
 *		be left floating, unlike many CMOS inputs.  
 *		The data sheets show them as not connected at all
 *		when in four bit mode.  It's not clear whether 
 *		these are driven or not when CPU reads from LCD
 *		controller.
 *
 *------------------------------------------------------------------------*/

volatile static bit     ENC_b   @ (unsigned)&PORTA*8+0;
volatile static bit     ENC_a   @ (unsigned)&PORTA*8+1;
volatile static bit     ENC_push @ (unsigned)&PORTA*8+2;
volatile static bit     DAC_cs 	@ (unsigned)&PORTA*8+3;
volatile static bit	AR_in	@ (unsigned)&PORTA*8+4;

#define ENC_AB_MASK     0b00000011

#define PORT_A_DIR      0b00010111      /* 1 = input bit */


volatile static bit     DDS_fq_ud @ (unsigned)&PORTB*8+0;
volatile static bit     LCD_rs  @ (unsigned)&PORTB*8+1;
volatile static bit     LCD_rw  @ (unsigned)&PORTB*8+2;
volatile static bit     LCD_e   @ (unsigned)&PORTB*8+3;

volatile static bit     LCD_d4  @ (unsigned)&PORTB*8+4;
volatile static bit	DAC_sclk @ (unsigned)&PORTB*8+4;

volatile static bit     LCD_d5  @ (unsigned)&PORTB*8+5;
volatile static bit     DDS_w_clk @ (unsigned)&PORTB*8+5;

volatile static bit     LCD_d6  @ (unsigned)&PORTB*8+6;
volatile static bit	DAC_din	@ (unsigned)&PORTB*8+6;

volatile static bit     LCD_d7  @ (unsigned)&PORTB*8+7;
volatile static bit     DDS_d7  @ (unsigned)&PORTB*8+7;

#define LCD_RS_MASK     0x02
#define LCD_RW_MASK     0x04
#define LCD_E_MASK      0x08

#define PORT_B_OUT      0b00000000      /* all output most of the time. */
#define PORT_B_IN4      0b11110000	/* top 4 bits input. */


/*
 * Set enc_sample when knob is rotated.
 */

volatile static signed char Twisted;



/*------------------------------------------------------------------------
 *
 * Name:        main
 *
 * Purpose:     Main program for Signal Generator.
 *
 * Global:	Twisted 	- Set by interrupt handler when
 *				  rotary encoder knob is twisted.
 *
 *
 * Description: Initialize the I/O devices and data.
 *		Wait for user input to change frequency.
 *                     
 *------------------------------------------------------------------------*/

void main ()
{
	byte_t freq[DIGITS];
	byte_t cursor_digit;
	
	
/*
 * Set up I/O ports.
 */
        PORTB = 0;
        TRISB = PORT_B_OUT;

        PORTA = 0;
        TRISA = PORT_A_DIR;


/*
 * Set up timer for interrupts of about 2400 Hz.
 */

 	asm("CLRWDT");		/* Manual says so. */

 	OPTION = 0;		/* Prescale of 2. */
 				/* which is really Fosc/8. */
 				/* Enable weak pullups on any */
 				/* port B inputs. */
 	INTCON = 0;
 	T0IE = 1;
 	GIE = 1;

/*
 * Initialize the DDS & DAC chips.
 */
        dds_init ();
        dac_init ();

/*
 * Initialize the LCD.
 */

        lcd_init ();

#if 1
        lcd_set_cursor (0);
        lcd_write_data ('S');
        lcd_write_data ('i');
        lcd_write_data ('g');
        lcd_write_data (' ');
        lcd_write_data ('G');
        lcd_write_data ('e');
        lcd_write_data ('n');
        lcd_write_data (' ');
        lcd_set_cursor (8);
        lcd_write_data ('2');
        lcd_write_data ('0');
        lcd_write_data ('0');
        lcd_write_data ('1');
        lcd_write_data (' ');
        lcd_write_data ('(');
        lcd_write_data ('a');
        lcd_write_data (')');
#endif
        
        delay(250);
        delay(250);
        delay(250);
        delay(250);


/*
 * Set to 137 Hz initially.
 */
 	bcd_clear (freq);
        freq[1] = 7;
        freq[2] = 3;
        freq[3] = 1;

#if DIGITS == 9
 	cursor_digit = 1;
#else
 	cursor_digit = 0;
#endif
 	 	
        lcd_set_cursor (0);
        lcd_write_data (' ');
        lcd_write_data (' ');
        lcd_write_data (' ');
        lcd_write_data (' ');
        
	bcd_display (freq, cursor_digit);
 	dds_from_bcd (freq);

/*
 * Wait for user or serial port input.
 */
 	Twisted = 0; 	
 	
        for (;;) {

/*
 * First priority: Process any commands from serial port.
 */

 	    if (ar_ch_avail()) {
		
	    	switch (ar_get_ch ()) {

		    case FREQUENCY_CMD:

		    	cmd_frequency (freq, cursor_digit);
		    	break;

		    case MESSAGE_CMD:

		    	cmd_message ();
		    	break;

                    case DAC_CMD:

                        cmd_dac ();
                        break;

                    case SWEEP_CMD:

                        cmd_sweep ();
                        break;

		    case NULL_CMD:

		    	break;
		}
    	
		continue;
	    }
	    
	    
/*
 * Next: Was knob twisted?
 */
 	    if (Twisted) {

	 	twist_event (freq, &cursor_digit);
	        continue;
 	    }

/*
 * Lower priority updating tasks.
 *
 * Not sure about the order.
 *
 * When getting commands from the serial port (e.g. sweep
 * generator) we want to update the output frequency quickly
 * and it's OK if the display trails behind.
 *
 * For interactive use, we want the display to be responsive
 * and it doesn't matter much if the resulting frequency out
 * takes a while to change.
 */
 	    /***** to be continued ******/
	    
        }
        
} /* end main */



/*------------------------------------------------------------------
 *
 * Name:        cmd_frequency
 *
 * Purpose:  	Allows computer to set the display and output frequency.   
 * 
 * Inputs:	freq		- Current frequency in BCD.
 * 		cursor_digit	- Position of cursor on display.
 *
 * Outputs:	freq		- Updated with data from serial port.
 *
 * Description:	Same effect as entering the frequency manually.
 *		The signal generator converts the decimal display to
 *		corresponding binary for the DDS chip.
 *
 *		See sg_cmd.h for more details.
 *
 * Assumptions:	9 digits will always be sent.
 *
 *------------------------------------------------------------------*/

void cmd_frequency (byte_t *freq, byte_t cursor_digit)
{
	byte_t n;

	n = DIGITS;
	while (n) {

	    n--;
	    freq[n] = ar_get_ch() & 0xF;
	}
		
	dds_from_bcd (freq);
	bcd_display (freq, cursor_digit);
}



/*------------------------------------------------------------------
 *
 * Name:        cmd_message
 *
 * Purpose:  	Allows computer to set the display.   
 * 
 * Inputs:	
 *
 * Outputs:	
 *
 * Description:	See sg_cmd.h for more details.
 *
 * Notes:	This carefully puts the cursor back where it 
 *		was before the command so manual inputs are not
 *		disrupted by computer displays in the first
 *		few character positions.
 *
 * Assumptions:	
 *
 *------------------------------------------------------------------*/

void cmd_message ()
{
	struct message_cmd_s c;
	byte_t save_cursor;

/*
 * Save current cursor position so it can be restored.
 */
 	while ( lcd_read_bf_ac() & 0x80) 
 	    ;
 	save_cursor = lcd_read_bf_ac();
 	if (save_cursor > 0x40) {
	    save_cursor += 8 - 0x40;
 	}
 	    
	c.position = ar_get_ch ();
	c.len = ar_get_ch ();
	c.reserved = ar_get_ch ();

	while (c.len--) {
/*
 * Setting cursor each time is a little wasteful but
 * it hides the annoying non-contiguous memory used for 
 * two pairs of 8 characters.
 */
 	    lcd_set_cursor (c.position++);
 	    lcd_write_data (ar_get_ch());
	}

	lcd_set_cursor (save_cursor);
}



/*------------------------------------------------------------------
 *
 * Name:        cmd_dac
 *
 * Purpose:     Allows computer to set the D/A converter outputs   
 * 
 * Inputs:
 *
 * Outputs:    
 *
 * Description: Set the DDS output amplitude and the two general
 *              purpose analog outputs.
 *
 *		See sg_cmd.h for more details.
 *
 *------------------------------------------------------------------*/

void cmd_dac ()
{
        struct dac_cmd_s c;

        c.address = ar_get_ch();
        c.dac_data = ar_get_ch();

        dac_write (c.address, c.dac_data);
}



/*------------------------------------------------------------------
 *
 * Name:        cmd_sweep
 *
 * Purpose:     Sweep frequency back and forth over specified range.   
 * 
 * Inputs:
 *
 * Outputs:    
 *
 * Description: 
 *
 *
 *              This will run "forever" unless stopped by
 *              (1) another command from serial port.
 *              (2) rotary encoder activity.
 *
 *
 *		See sg_cmd.h for more details.
 *
 *------------------------------------------------------------------*/

#define Z_SIZE 51       /* Variable later??? */

void cmd_sweep ()
{
        struct sweep_cmd_s cmd;
        byte_t *p;
        byte_t x, m, s;     
        byte_t x_step;

/*
 * Read rest of command into structure.
 */
        p = (byte_t *)(&cmd);
        m = sizeof(cmd);
        do {
            *p++ = ar_get_ch();
        } while (--m);


        x = 0;
        x_step = 1;

        while (1) {           

            m = cmd.n_marks - 1;
            do {

                s = cmd.spacing;
                do {

                    long5_add (cmd.lower_f, cmd.f_step);

                    dds_write_freq (cmd.lower_f + 1);

                    if (s != 1) {
                        dac_write (DAC_Z, 0);
                    }

                    x += x_step;

                    dac_write (DAC_X, x);

                    if (s == 1) {
                        dac_write (DAC_Z, Z_SIZE);
                    }

                } while (--s);

            } while (--m);

            long5_negate (cmd.f_step);
            x_step = ( - x_step);

/*
 * Check for termination conditions.
 */
            if (Twisted != 0)
                return;

            if ( ! ENC_push)
                return;

            if (ar_ch_avail())
                return;
        }
}




/*------------------------------------------------------------------
 *
 * Name:        twist_event
 *
 * Purpose:  	Handle the rotary encoder rotation events.   
 * 
 * Inputs:	freq		- Current frequency in BCD.
 * 		cursor_digit	- Position of cursor on display.
 *
 * Outputs:	freq		- Updated if knob not pushed in.
 *		cursor_digit	- Updated if knob is pushed in.
 *
 * Global In:	Twisted		- Positive if twisted to the right.
 *				  Negative if twisted to the left.
 *
 * Global Out:	Twisted		- Decremented by 1 toward zero.
 *
 *
 * Description:	To be continued ...
 *
 * Assumptions:	Will be called only if Twisted is non-zero.
 *
 *------------------------------------------------------------------*/


void twist_event (byte_t *freq, byte_t *cursor_digit)
{
	
	if ( ! ENC_push) {

/*
 * Knob is pushed in - change cursor position.
 */
	    if (Twisted > 0) {		/* Clockwise. */
	    				/* Right toward less significant. */
	        if (*cursor_digit > 0)
	            (*cursor_digit)--;
	        Twisted--;
	    }
	    else {
		if (*cursor_digit < DIGITS - 1)     
	    	    (*cursor_digit)++;
	        Twisted++;
	    }
	    
	    bcd_display (freq, *cursor_digit);

 	}
 	else {

/*
 * Not pushed in - change selected digit.
 */
	    if (Twisted > 0) {		/* Clockwise. */
	    	bcd_incr (freq, *cursor_digit);
	        Twisted--;
	    }
	    else {
	        bcd_decr (freq, *cursor_digit);
	        Twisted++;
	    }
	    
	    bcd_display (freq, *cursor_digit);
	    dds_from_bcd (freq);
	    
	}
}


/*------------------------------------------------------------------
 *
 * Name:        bcd_clear
 *
 * Purpose:     Set binary coded decimal string to zero.
 *
 * Inputs:      bcd	- Pointer to array of DIGITS bytes.
 *
 * Outputs:	Each element of bcd array is set to zero.
 *
 *------------------------------------------------------------------*/

void bcd_clear (byte_t *bcd)
{
	byte_t n;

	for (n=0; n<DIGITS; n++) {
	    bcd[n] = 0;
	}
}


/*------------------------------------------------------------------
 *
 * Name:        bcd_incr
 *
 * Purpose:     Increment specified digit of BCD array.
 *
 * Inputs:      bcd	- Pointer to array of DIGITS bytes.
 *		position - Index of digit to increment.
 *			0 for the least significant byte.
 *
 * Outputs:	Digit(s) in BCD array are modified.
 * 
 * Description: If a digit overflows past 9, it is set to 0 and
 *		the next higher digit is incremented.
 *		Stop if number is >= 20,000,000.
 *
 * Example for DIGITS == 8:
 *	
 *		Initial contents of bcd: 	0 0 9 0 0 0 0 0
 *		which means 900 Hz.
 *		Position is 2 which means add 100 Hz.
 *
 *		Resulting contents of bcd:	0 0 0 1 0 0 0 0
 *
 * Example for DIGITS == 9:
 *	
 *		Initial contents of bcd: 	0 0 9 0 0 0 0 0 0
 *		which means 900.0 Hz.
 *		Position is 3 which means add 100 Hz.
 *
 *		Resulting contents of bcd:	0 0 0 1 0 0 0 0 0
 *
 *------------------------------------------------------------------*/


void bcd_incr (byte_t *bcd, byte_t position)
{
	for ( ; position < DIGITS; position++) {
	    if (++bcd[position] <= 9) {
		break;
	    }
	    bcd[position] = 0;
	}

	if (bcd[DIGITS-1] >= 2) {
	    bcd_clear (bcd);		/* Don't go beyond 20 MHz. */
	    bcd[DIGITS-1] = 2;
	}
}


/*------------------------------------------------------------------
 *
 * Name:        bcd_decr
 *
 * Purpose:     Decrement specified digit of BCD array.
 *
 * Inputs:      bcd	- Pointer to array of DIGITS bytes.
 *		position - Index of digit to decrement.
 *			0 for the least significant byte.
 *
 * Outputs:	Digit(s) in BCD array are modified.
 * 
 * Description: If a digit underflows below 0, it is set to 9 and
 *		the next higher digit is decremented.
 *		Stop if number reaches zero.
 *
 * Example for DIGITS == 8:	
 *
 *		Initial contents of bcd: 	0 0 0 1 0 0 0 0
 *		which means 1000 Hz.
 *		Position is 2 which means subtract 100.0 Hz.
 *
 *		Resulting contents of bcd:	0 0 9 0 0 0 0 0
 *
 * Example for DIGITS == 9:	
 *
 *		Initial contents of bcd: 	0 0 0 1 0 0 0 0 0
 *		which means 1000.0 Hz.
 *		Position is 3 which means subtract 100.0 Hz.
 *
 *		Resulting contents of bcd:	0 0 9 0 0 0 0 0 0
 *
 *------------------------------------------------------------------*/


void bcd_decr (byte_t *bcd, byte_t position)
{
	for ( ; position < DIGITS; position++) {
	    if (bcd[position] > 0) {
		bcd[position]--;
		return;
	    }
	    bcd[position] = 9;
	}

	bcd_clear (bcd);	/* Oops, no more digits to "borrow from." */
				/* In other words we went past zero. */
}



/*------------------------------------------------------------------
 *
 * Name:        bcd_display
 *
 * Purpose:     Display number on the LCD.
 *
 * Inputs:      bcd	- Pointer to array of DIGITS bytes.
 *		cursor_digit - Index of digit to highlight with cursor.
 *			0 for the least significant byte.
 *
 * Outputs:	Digits are written to LCD.
 *		For readability, leading zeros are normally
 *		blanked and commas are inserted.
 * 
 * Example:	BCD array in:  		4 3 2 1 0 0 0 0
 *		cursor digit:	 	1
 *
 *		Displayed as:		1,234
 *					   ^
 *
 *------------------------------------------------------------------*/


void bcd_display (byte_t *bcd, byte_t cursor_digit)
{
/*
 * Position on LCD for each of the digits.
 * They are not adjacent because commas are inserted.
 */
#if DIGITS == 9
 	static const byte_t position_of_digit[DIGITS] = 
	 	{ 15,   13, 12, 11,   9, 8, 7,   5, 4 };
#else
 	static const byte_t position_of_digit[DIGITS] = 
	 	{ 15, 14, 13,   11, 10, 9,   7, 6 };
#endif
	 	
/*
 * Character, if any, to display after the digit.
 */
#if DIGITS == 9
 	static const byte_t append_ch[DIGITS] = 
 		{ 0, '.', 0, 0,   ',', 0, 0,   ',', 0 };
#else
 	static const byte_t append_ch[DIGITS] = 
 		{ 0, 0, 0,   ',', 0, 0,   ',', 0 };
#endif
 		
	byte_t leading_zero_blank;
#undef  SMALLER 			/* Turns out to be one byte */
					/* larger.  TBD remove. */
#if SMALLER
	byte_t d;
#else
	signed char d;			/* Signed for while >= 0 loop. */
					/* Rewrite for efficiency??? */
#endif

	leading_zero_blank = TRUE;
	lcd_hide_cursor ();

#if SMALLER
	d = DIGITS;
	do {
	    d--;
#else
	for (d = DIGITS-1; d >= 0; d--) {
#endif
	    lcd_set_cursor (position_of_digit[d]);

/*
 * Don't blank out digit at cursor.
 */
	    if (d == cursor_digit)
	        leading_zero_blank = FALSE;

/*
 * Don't blank out units (1 Hz) digit.
 */
#if DIGITS == 9                 /* LSB is 0.1 Hz. */
            if (d == 1)
	        leading_zero_blank = FALSE;
#else
            if (d == 0)
	        leading_zero_blank = FALSE;
#endif

/*
 * Non-zero digit found: stop blanking.
 */
	    if (bcd[d]) 
	        leading_zero_blank = FALSE;

	    if (leading_zero_blank) 
	 	lcd_write_data (' ');
	    else 
		lcd_write_data (bcd[d] + '0');

	    /* Blank any ".0" if cursor not set to last digit. */

#if DIGITS == 9
	    if (d == 1 && cursor_digit != 0 && bcd[0] == 0) {
                leading_zero_blank = TRUE;
	    }
#endif
	    if (append_ch[d]) {
		lcd_set_cursor (position_of_digit[d] + 1);
		if (leading_zero_blank)
		    lcd_write_data (' ');
		else
		    lcd_write_data (append_ch[d]);
	    }
#if SMALLER
	} while (d != 0);
#else
	}
#endif

	lcd_set_cursor (position_of_digit[cursor_digit]);
	lcd_show_cursor ();	
}



/*------------------------------------------------------------------
 *
 * Name:        long5_add
 *
 * Purpose:     Add two five byte numbers. 
 *
 * Inputs:      psum, padd      - Add these two numbers.
 *                                Actually pointers to LSB.
 *
 * Outputs:     psum            - Their sum goes here.
 *
 * Description: It would have been a lot easier to use 4 byte
 *              numbers for all the calculations because the compiler
 *              has a built in "long" integer type.
 *              However, there would be excessive accumulation of
 *              round off errors when doing a lot of adding, especially
 *              for the sweep function.
 *
 *------------------------------------------------------------------*/

void long5_add (byte_t *psum, byte_t *padd)
{
        byte_t i;               /* Counter for looping. */
        byte_t kerry;           /* Carry from one byte to next. */
	byte_t x;		/* Partial intermediate result. */

        kerry = 0;

        i = DDS_PREC;
        do {
/*
 * Apply any carry from preceding byte position.
 */
            x = *psum + kerry;

	    if (CARRY) {
		kerry = 1;	/* Will carry into next byte. */
	    }
	    else {
	        kerry = 0;	/* Didn't overflow. */
	    }
/*
 * Add bytes; keep track of any additional carry.
 */
 	    /* Compiler error: *psum += *padd */
 	    
            *psum = x + *padd;

            if (CARRY) {
                kerry++;
            }

            psum++;
            padd++;

        } while (--i); 
}



/*------------------------------------------------------------------
 *
 * Name:        long5_negate
 *
 * Purpose:     Negate a five byte numbers. 
 *
 * Inputs:      pnum            - Pointer to LSB of 5 bytes.
 *
 * Outputs:     pnum            - Result goes into same place.
 *
 *------------------------------------------------------------------*/

void long5_negate (byte_t *pnum)
{
        byte_t i;               /* Counter for looping. */
        byte_t kerry;           /* Carry from one byte to next. */
	

        kerry = 1;

        i = DDS_PREC;
        do {
            *pnum = ( ~ *pnum) + kerry;

	    if (CARRY) {
		kerry = 1;	/* Will carry into next byte. */
	    }
	    else {
	        kerry = 0;	/* Didn't overflow. */
	    }
	    
            pnum++;

        } while (--i);

}



/*------------------------------------------------------------------------
 *
 * Name:        data_eeprom_read
 *
 * Purpose:     Read a byte from non-volatile data memory in the 16C84.
 *
 * Inputs:      adr     - Address in range of 0 to 63.
 *
 * Returns:     A byte from the specified location.
 *
 * Assumptions: Input address is in correct range.
 *              On the 16C84 the two high order bits are ignored.
 *
 * Description: Basically, set the EEADR register and read from EEDATA.
 *
 *
 *------------------------------------------------------------------------*/

#if 0

        /* Use the in-line EEPROM_READ macro instead. */

byte_t data_eeprom_read (byte_t adr)
{
        EEADR = adr;
        RD = 1;                 /* Part of EECON1. */
        return (EEDATA);
}

#endif

/*------------------------------------------------------------------------
 *
 * Name:        data_eeprom_write
 *
 * Purpose:     Write a byte to non-volatile data memory.
 *
 * Inputs:      adr     - Address in range of 0 to 63.
 *              dat     - Data byte in range of 0 to 255.
 *
 * Outputs:     Write to non-volatile memory.
 *
 * Returns:     None.
 *
 * Assumptions: Address is within range.
 *              This program does not use interrupts.
 *
 * Description: This is a lot more complicated than read to
 *              avoid accidental writes.
 *              Set the EEADR and EEDATA registers.
 *              Write certain patterns to EECON2.
 *              Wait for completion which is typically 10 mS.
 *
 *------------------------------------------------------------------------*/

#ifdef NOT_YET

void data_eeprom_write (byte_t adr, byte_t dat)
{
        EEADR = adr;
        EEDATA = dat;
        /* GIE = 0; if using interrupts. */
        WREN = 1;
        EECON2 = 0x55;
        EECON2 = 0xaa;
        WR = 1;
        WREN = 0;
        /* GIE = 1; if using interrupts. */
        while (WR)
            ;
        EEIF = 0;
}

#endif


/*------------------------------------------------------------------------
 *
 * Name:        delay
 *
 * Purpose:     Delay specified number of milliseconds.
 *		Not tested or guaranteed to be at all accurate.
 *
 *------------------------------------------------------------------------*/

void delay (byte_t ms)
{
        byte_t i;

        while (ms > 0) {

/*
 * Assuming 4 MHz clock, 1 instruction = 1 uS.
 * Loop has overhead of 3 instructions each time around.
 * One more instruction (NOP) should give total of 4 instructions
 * each time around the inner loop.
 */

#if XTAL == 4000000
            i = 249;
#elif XTAL == 2457600
            i = 154;
#else
#error Unexpected XTAL
#endif
            
            do {
                NOP;
            } while (--i);

            ms--;
        }
}



/*------------------------------------------------------------------
 *
 * Name:        lcd_init
 *
 * Purpose:     Initialize the LCD.
 *
 * Description: There are a couple tricky things here.
 *              (1) A one line x 16 character display is wired
 *                  as two lines of 8 characters.
 *                  The second half begins at address 0x40.
 *              (2) We only have a 4 bit data path.  We start in
 *                  8 bit mode and switch to 4 bit mode.
 *
 *------------------------------------------------------------------*/

void lcd_init ()
{

/*
 * I don't understand the reason for the same command three
 * times with different delays but that's what the data
 * sheet says.
 */
        delay (60);             /* >= 15 mS. */
        lcd_put4 (3);
        delay (16);             /* >= 4.1 mS. */
        lcd_put4 (3);
        delay (1);              /* >= 0.1 mS. */
        lcd_put4 (3);
        delay (16);             /* >= 4.1 mS. */
        lcd_put4 (2);           /* Enter 4 bit data mode. */
        delay (1);

/*
 * We are now in 4 bit mode and must do writes in pairs.
 */
        lcd_put4 (2);           /* 4 bit mode again. */
        lcd_put4 (8);           /* 2 line display, 5x7 characters. */
        delay (1);

        lcd_put4 (0);           /* Display off. */
        lcd_put4 (8);
        delay (1);

        lcd_put4 (0);           /* Display on, Cursor and blinking off. */
        lcd_put4 (12);
        delay (3);              /* >= 1.64 mS. */

        lcd_put4 (0);           /* Cursor right automatically. */
        lcd_put4 (6);
        delay (1);
}


/*------------------------------------------------------------------
 *
 * Name:        lcd_put4
 *
 * Purpose:     Send a nybble to the LCD with instruction mode.
 *              This is used only during initialization.
 *
 * Inputs:      x       - Only 4 low order bits used.
 *
 * Description: Write the data with,
 *                RS = 0 for instruction
 *                RW = 0 for write
 *              Pulse E.
 *
 * Note:        Initially we are in 8 bit mode and we only
 *              write the high order half of a full byte.
 *              After switching to 4 bit mode, this must be
 *              called in pairs, high order then low order half.
 *
 *------------------------------------------------------------------*/

void lcd_put4 (byte_t x)
{
        PORTB = x << 4;
        NOP;
        LCD_e = 1;             
        NOP;
        LCD_e = 0;              
}


/*------------------------------------------------------------------
 *
 * Name:        lcd_read_bf_ac
 *
 * Purpose:     Read busy flag and address counter.
 *
 * Returns:	A byte of the following format:
 *
 *			B A A A A A A A
 *
 *				B = 1 for busy
 *				    0 for ready
 *
 *			        AAAAAAA	= address counter
 *
 * Description:	The 4 high bits of port B are temporarily set to
 *		input rather than their normal output.
 *
 *------------------------------------------------------------------*/

byte_t lcd_read_bf_ac (void)
{
	byte_t result;

	TRISB = PORT_B_IN4;

	PORTB = LCD_RW_MASK;	/* RS = 0 ; R/-W = 1 */
	NOP;
	LCD_e = 1;
	NOP;
	result = PORTB & 0xF0;
	LCD_e = 0;

	LCD_e = 1;
	NOP;
	result |= PORTB >> 4;
	LCD_e = 0;

	TRISB = PORT_B_OUT;

	return (result);	
}
	

/*------------------------------------------------------------------
 *
 * Name:        lcd_write_data
 *
 * Purpose:     Send a character to display.
 *
 * Inputs:      x       - An ASCII character.
 *
 * Description: Write the data as two nybbles with
 *                RS set to 1 for data
 *                RW set to 0 for write
 *
 *------------------------------------------------------------------*/

void lcd_write_data (byte_t x)
{
	while (lcd_read_bf_ac() & 0x80)
	    ;
	    
        PORTB = (x & 0xf0) | LCD_RS_MASK;
        NOP;
        LCD_e = 1;             
        NOP;
        LCD_e = 0;              

        PORTB = (x << 4) | LCD_RS_MASK;
        NOP;
        LCD_e = 1;             
        NOP;
        LCD_e = 0;
}



/*------------------------------------------------------------------
 *
 * Name:        lcd_write_instr
 *
 * Purpose:     Send an instruction to display.
 *
 * Inputs:      x       - Instruction code.
 *
 * Description: Write the data as two nybbles with
 *                RS set to 0 for instruction
 *                RW set to 0 for write
 *
 * 		The application shouldn't use this function.
 *		Instead it should use functions such as 
 *		lcd_set_cursor.
 *
 *------------------------------------------------------------------*/

void lcd_write_instr (byte_t x)
{
	while (lcd_read_bf_ac() & 0x80)
	    ;

        PORTB = x & 0xf0;
        NOP;
        LCD_e = 1;             
        NOP;
        LCD_e = 0;              

        PORTB = x << 4;
        NOP;
        LCD_e = 1;             
        NOP;
        LCD_e = 0;
}



/*------------------------------------------------------------------
 *
 * Name:        lcd_set_cursor
 *
 * Purpose:     Set memory address in preparation for a data write
 *		or displaying the cursor.
 *
 * Inputs:      position	- 0 for left .. 15 for right edge.
 *
 * Description: Note that a 16 character display has memory organized
 *		as two non-adjacent 8 character areas.
 *		This function hides that detail so the rest of the
 *		application doesn't have to be concerned about it.
 *
 *		The commands to set the address are:
 *
 *              1 0 0 0 0 n n n         for first 8 characters
 *              1 1 0 0 0 n n n         for second 8 characters.
 *
 *------------------------------------------------------------------*/

void lcd_set_cursor (byte_t position)
{
	if (position < 8) {
	    lcd_write_instr (0x80 | position);
	}
	else {
	    lcd_write_instr (0xc0 | (position & 0x7));
	}
}


/*------------------------------------------------------------------
 *
 * Name:        lcd_hide_cursor
 *		lcd_show_cursor
 *
 * Purpose:     Turn cursor display off / on.   
 *
 *------------------------------------------------------------------*/

void lcd_hide_cursor (void)
{
	lcd_write_instr (0b00001100);
}

#if 1			/* Using underscore cursor. */
			/* Requires at least 8 vertical dots */
			/* per character.  Some have only 7. */
 
void lcd_show_cursor (void)
{
	lcd_write_instr (0b00001110);
}

#else			/* Using blinking. */
 
void lcd_show_cursor (void)
{
	lcd_write_instr (0b00001101);
}

#endif


/*------------------------------------------------------------------
 *
 * Name:        dds_init
 *
 * Purpose:     Initialize the DDS chip.   
 *
 * Description: Power on default is parallel mode.
 *              Set serial interface mode.
 *              (Set frequency to zero. - removed)
 *
 * Asumptions:  The data bits are hardwired to xxxxx011.
 *
 *------------------------------------------------------------------*/

void dds_init ()
{
	/* byte_t n; */
	
        DDS_fq_ud = 0;
        NOP;
        DDS_w_clk = 0;
        NOP;

        DDS_w_clk = 1;
        NOP;
        DDS_w_clk = 0;
        NOP;

        DDS_fq_ud = 1;
        NOP;
        DDS_fq_ud = 0;

}



/*------------------------------------------------------------------
 *
 * Name:        dds_from_bcd
 *
 * Purpose:     Convert frequency, in BCD, to binary then send
 *		it to the DDS chip.  
 *
 * Inputs:      bcd	- Pointer to array of 'DIGITS' digits.     
 *
 * Description: Convert from BCD to binary by multiplying each
 *		digit by an appropriate constant and adding them
 *		all up. 
 *		These constants are kept in the data EEPROM
 *		so recalibration is possible without replacing
 *		the program.
 *
 *------------------------------------------------------------------*/

/* define DEBUG_F 1 */	/* Look at what is sent to DDS chip. */

#if DEBUG_F
const byte_t hexc[16] = "0123456789abcdef";
#endif

void dds_from_bcd (byte_t *bcd)
{

#if 1 /* new way - break out long5_add function */

        byte_t weight[DDS_PREC]; /* Weight of one decimal digit. */
	byte_t d;		/* Index into BCD digits. */
        byte_t i;               /* Looping index. */

	byte_t sum[DDS_PREC];	/* Result of multiplies and adds. */
	
#if DEBUG_F
	byte_t k;
#endif
	
/* 
 * To minimize truncation/roundoff errors, intermediate calculations
 * are performed with 5 bytes of precision.  Four bytes represent
 * whole numbers and the fifth is a fractional part.  See the 
 * companion weight.c program for more information.
 */
        sum[0] = 128;           /* +1/2 for rounding. */
        sum[1] = sum[2] = sum[3] = sum[4] = 0;

	EEADR = 0;		/* Weighting factors from here. */

        for (d=0; d<DIGITS; d++) {

/*
 * Get weighting factor for a digit of display.
 */
            for (i=0; i<DDS_PREC; i++) {
                RD = 1;         /* Read from data EEPROM. */
                NOP;
                weight[i] = EEDATA;
                EEADR++;
            }
/*
 * Multiply it by the digit value.
 */
            if ((i = bcd[d]) != 0) {
                do {
                    long5_add (sum, weight);
                } while (--i);
            }
	
        }

#else   /* OLD way - all inline code. *

	byte_t sum[DDS_PREC];	/* Result of multiplies and adds. */
	byte_t *ps;		/* Pointer to element of above. */ 
	byte_t k;		/* Counter for looping thru above. */
	
	byte_t d;		/* Index into BCD digits. */
	byte_t i;		/* For multiplying by digit value. */
	byte_t kerry;		/* Carry from one byte to next. */
	

/* 
 * To minimize truncation/roundoff errors, intermediate calculations
 * are performed with 5 bytes of precision.  Four bytes represent
 * whole numbers and the fifth is a fractional part.  See the 
 * companion weight.c program for more information.
 */
	sum[0] = sum[1] = sum[2] = sum[3] = sum[4] = 0;

	kerry = 128;		/* + 1/2 for rounding. */
	EEADR = 0;		/* Weighting factors from here. */

	k = DDS_PREC;
	ps = sum;
	do {
	
/*
 * Apply any carry from preceding byte position.
 */
	    *ps += kerry;
	    if (CARRY) {
		kerry = 1;	/* Will carry into next byte. */
	    }
	    else {
	        kerry = 0;	/* Didn't overflow. */
	    }

	    for (d=0; d<DIGITS; d++) {
	        if (i = bcd[d]) {
		    RD = 1;	/* Read from data EEPROM. */
		    do {
			*ps += EEDATA;
			if (CARRY) {
			    kerry++;
			}
		    } while (--i);
	        }
	        EEADR++;	/* Select next weighting constant. */
	    }
	    ps++;
	
	} while (--k);


#endif


/*
 * Discard the fractional bits and use only the whole number.
 */
#if DEBUG_F
        lcd_set_cursor (0);
        for (k=4; k>0; k--) {
	    lcd_write_data(hexc[sum[k] >> 4]);
	    lcd_write_data(hexc[sum[k] & 0xf]);
        }
#endif
 	/* dds_write_freq (&sum[1]); */
	dds_write_freq (&sum[1]);
	
}



/*------------------------------------------------------------------
 *
 * Name:        dds_write_freq
 *
 * Purpose:     Send frequency control information to DDS chip.  
 *
 * Inputs:      f       - Pointer to LSB of 4 data bytes.     
 *
 * Description: Shift out 4 bytes of frequency and 1 of phase/control.
 *
 *------------------------------------------------------------------*/

 
void dds_write_freq (byte_t *f)
{
        byte_t n;

        
        DDS_w_clk = 0;

        n = 4;
        do {
            dds_shift_byte (*f);
            f++;
        }
	while (--n);
	
        dds_shift_byte (0);

        DDS_fq_ud = 1;
        NOP;
        DDS_fq_ud = 0;
}



/*------------------------------------------------------------------
 *
 * Name:        dds_shift_byte
 *
 * Purpose:     Shift out one byte of data to DDS chip. 
 *
 * Inputs:      x       - One byte of data.
 *
 * Description: Send it out LSB first.
 *              Generally this will be called 5 times followed
 *              by a pulse of FQ_UD to update the frequency.
 *
 *------------------------------------------------------------------*/

void dds_shift_byte (byte_t x)
{
        byte_t n;

     
        n= 8;
        do {
	    DDS_d7 = 0;
            if (x & 1)
                DDS_d7 = 1;

            x >>= 1;

            DDS_w_clk = 1;
            NOP;
            DDS_w_clk = 0;

        } while (--n);
}


/*------------------------------------------------------------------
 *
 * Name:        dac_init
 *
 * Purpose:     Initialize the D/A converter. 
 *
 * Inputs:      None.
 *
 * Description: Power on reset should set all the outputs to zero.
 *              Just make sure that the chip select is 1.
 *
 *------------------------------------------------------------------*/

void dac_init ()
{
        DAC_cs = 1;
}


/*------------------------------------------------------------------
 *
 * Name:        dac_write
 *
 * Purpose:     Set an output of the quad D/A converter.
 *
 * Inputs:      addr    - Desired output.  One of:
 *                              DAC_A - Amplitude, 0=maximum.
 *                              DAC_B - Currently not used.
 *                              DAC_C, DAC_X - Sweep X (green).
 *                              DAC_D, DAC_Z - Sweep marker (yellow).
 *
 *              value   - Data for the DAC, 0 - 255.
 *
 * Description: 
 *
 *------------------------------------------------------------------*/

#define DSO 1				/* Code is 6 instructions */
 					/* shorter but slightly slower */
 					/* due to 4 useless bits */
 					/* being shifted out. */

#ifdef DSO
void dac_shift_out (byte_t x)
{
	byte_t n;
	
        n= 8;
        do { 
	    DAC_din = 0;
            if (x & 0x80)
                DAC_din = 1;
            x <<= 1;
            DAC_sclk = 1;
            NOP;
            DAC_sclk = 0;
        } while (--n);
}
#endif


void dac_write (enum dac_addr_e addr, byte_t value)
{
#ifdef DSO
        DAC_sclk = 0;                   /* Shared pin. */
                                        /* Condition unpredictable. */
                                        
                                        /* No NOP because they are */
                                        /* on different ports. */
                                        
        DAC_cs = 0;                     /* Enable shift register. */


        dac_shift_out ((addr << 2) | 3);
        dac_shift_out (value);

        DAC_cs = 1;			/* Execute command. */
        
#else

        byte_t cmd;         
        byte_t n; 

        
        DAC_sclk = 0;                   /* Shared pin. */
                                        /* Condition unpredictable. */
        cmd = (addr << 2) | 3;

        DAC_cs = 0;                     /* Enable shift register. */

        n = 4;
        do {
	    DAC_din = 0;
            if (cmd & 0x08)
                DAC_din = 1;
            cmd <<= 1;
            DAC_sclk = 1;
            NOP;
            DAC_sclk = 0;
        } while (--n);

        n= 8;
        do { 
	    DAC_din = 0;
            if (value & 0x80)
                DAC_din = 1;
            value <<= 1;
            DAC_sclk = 1;
            NOP;
            DAC_sclk = 0;
        } while (--n);

        DAC_cs = 1;                     /* Execute command. */
#endif
}

                               

/*------------------------------------------------------------------
 *
 * Name:        timer
 *
 * Purpose:     Handle timer interrupts of about 2400 Hz.
 *
 * Description: Sample the serial port and rotary encoder periodically.
 *
 * Outputs:	See ar_sample and enc_sample.
 *
 * Details:	The prescaler was set to "2" which is really
 *		oscillator divided by 4 then by 2.
 *
 *		For 4 MHz clock, 
 *			4000000 / 8 / 207 = about 2415.
 *
 *		For 2.4576 MHz, 
 *			2457600 / 8 / 128 = 2400.
 *
 *		Some fudging is needed due to the interrupt 
 *		latency time.
 *
 *		Too bad we can't get a prescale of 1 so we could
 *		just let the timer go all the way around and get
 *		an exact 2400.
 *		Could use 4.9152 or 9.8304 MHz crystal to get it exact.
 *
 *------------------------------------------------------------------*/

void interrupt timer (void)
{
#if XTAL == 4000000
	TMR0 = 256 - 198;		/* 256-198  ->  2404.2 Hz. */
					/* 256-197  ->  2415.9 Hz. */
				
#elif XTAL == 2457600
	TMR0 = 256 - 118;		/* 256-118  ->  2400.2 Hz */
					/* 256-126  ->  2259 Hz */
					/* 256-127  ->  2242 Hz */
#else
#error Unexpected XTAL.
#endif
	T0IF = 0;


/*
 * As a temporary test, toggle pin 2 just so we
 * can see if the timing is correct.
 * We want 1200 Hz on the frequency counter which
 * corresponds to a 2400 Hz interrupt rate.
 */

#if 0
 	if (LCD_pwr)
 	    LCD_pwr = 0;
 	else
 	    LCD_pwr = 1;
#endif
 	    
/*
 * I'm concerned about staying in here too long
 * so we'll ignore the rotary encoder when receiving
 * a character thru the RS-232 port.
 */

 	if ( ! ar_sample()) {
	 	
	    enc_sample ();
 	}
}


/*------------------------------------------------------------------
 *
 * Name:        ar_sample
 *
 * Purpose:     This is a software implementation of the R
 *		part of a UART and it's not universal, leaving
 *		only A and R for the name.
 *
 * Description: Sample the serial port 8x the bit rate.
 *		Gather any data arriving at 300 baud.
 *
 * Outputs:	Characters are stored in a queue in case the
 *		main loop can't process them fast enough.
 * 
 * Returns:	TRUE if receiving is in progress.
 *		FALSE if sitting idle.
 *
 *
 * Details:	The current state is in the variable ar_state.
 *		It is incremented for each sample.
 *
 *		0 	= sitting idle.
 *		1 	= beginning of start bit detected.
 *		4 	= middle of start bit.
 *		8n+12	= middle of bit n, 0-7
 *		76	= middle of stop bit.
 *
 *------------------------------------------------------------------*/
 
#define AR_Q_LEN 8

static byte_t ar_queue[AR_Q_LEN];
static byte_t ar_q_insert = 0;
static byte_t ar_q_remove = 0;
volatile static byte_t ar_q_count = 0;
 

byte_t ar_sample (void)
{
	static byte_t ar_state = 0;
	static byte_t rec_ch;

	
	if (ar_state == 0) {		
/*
 * Sitting idle; look for start bit.
 */
 	    if (AR_in) {
	        return (FALSE);
 	    }
 	    
	    ar_state = 1;
	    rec_ch = 0;
	    return (TRUE);
	}

/*
 * Reception is in progress.
 * Examine only the middle of each bit position.
 */
	if ((ar_state & 0b111) == 0b100) {

	    if (ar_state == 4) {
/*
 * Start bit.
 * Make sure it is still there.
 */
 		if (AR_in) {
	 	    ar_state = 0;
	 	    return (TRUE);
 		}
	    }
	    else if (ar_state == 76) {
/*
 * Stop bit.
 * Add result to queue if not framing error.
 */
		if (AR_in) {

		    if (ar_q_count < AR_Q_LEN) {
			ar_queue[ar_q_insert] = rec_ch;
			ar_q_insert++;
			if (ar_q_insert >= AR_Q_LEN) {
			    ar_q_insert = 0;
			}
			ar_q_count++;
		    }
		}
 		ar_state = 0;
 		return (TRUE);
	    }
	    else {
/*
 * One of the data bits.
 */
 		rec_ch >>= 1;
 		if (AR_in) {
	 	    rec_ch |= 0x80;
 		}
	    }
	} 	

	ar_state++;
	return (TRUE);
}



/*------------------------------------------------------------------
 *
 * Name:        ar_ch_avail
 *
 * Purpose:     See if a character is available.
 *
 * Returns:	Non-zero if data is available and ar_get_ch can be called.
 *		(Actually number of characters in received queue.)
 *
 * Global:	ar_q_count, and friends, above.
 *
 *------------------------------------------------------------------*/

byte_t ar_ch_avail (void)
{
	return (ar_q_count);
}


/*------------------------------------------------------------------
 *
 * Name:        ar_get_ch
 *
 * Purpose:     Read from the received character queue.
 *		Waits if none available.
 *
 * Returns:	Next character from queue or '?' if empty.
 *
 * Global:	ar_queue, and friends, above.
 *
 * Description:	Normally ar_ch_avail should be called first to see if
 *		a character is available to avoid waiting here forever.
 *		
 *------------------------------------------------------------------*/

byte_t ar_get_ch (void)
{
	byte_t result;

	while (ar_q_count == 0) 
	    ;
	    
	result = ar_queue[ar_q_remove];
	ar_q_remove++;
	if (ar_q_remove >= AR_Q_LEN) {
	    ar_q_remove = 0;
	}
	ar_q_count--;
	return (result);
}

	    
/*------------------------------------------------------------------
 *
 * Name:        enc_sample
 *
 * Purpose:     Sample the rotary encoder and look for rotation.
 *
 * Inputs:	Two bits from an input port.
 *
 * Gobal Out:	Twisted		- which is used by main loop.
 *				  Increment for right rotation.
 *				  Decrement for left rotation.
 *
 *------------------------------------------------------------------*/

#define MIN_STABLE 24
 
void enc_sample (void)
{
	byte_t sample;		
	static byte_t prev_sample;
	static byte_t stable_count;
	byte_t debounced;
	static byte_t prev_debounced;
	byte_t rotation;
	

	sample = PORTA & ENC_AB_MASK;
/*
 * Debounce.
 * Don't use the value until it has been stable for
 * at least 1/100 second.  (24 samples in this case.)
 */
 
	if (sample == prev_sample) {
	    if (stable_count < MIN_STABLE) {
		stable_count++;
		if (stable_count == MIN_STABLE) {
/*
 * Convert from Gray code to binary.
 */
		    debounced = sample ^ (sample >> 1);
		}
	    }
	    else {
		return;
	    }
	} 
	else {
	    stable_count = 0;
	    prev_sample = sample;
	    return;
	}

/*
 * Was there any change?
 * Careful!  In some cases you get the expected +1 or -1
 * but only the two low order bits are valid here.
 */
 	rotation = debounced - prev_debounced;

 	if (rotation) {
	    if (rotation & 2) {
	        Twisted++;
	    }
	    else {
		Twisted--;
	    }
	}

	prev_debounced = debounced;
}
        
/* end DDS.C */

