/*--------------------------------------------------------------------
 *
 * Name:        SG_CMD.H
 *
 * Purpose:     Things in common for signal generator application
 *              and other associated applications.
 *
 *
 * History:     Release 1, July 2001
 *
 *--------------------------------------------------------------------*/


typedef unsigned char byte_t;


/*
 * Oscillator frequency for the DDS chip.
 */

#define DDS_CLOCK 66.666e6


/*
 * Number of digits in display.
 *
 * Originally I tried 8 to allow up to 20,000,000 in 1 Hz steps.
 * Later used 9 to allow 0.1 Hz resolution.
 */

#define DIGITS 9


/*
 * Number of bytes used to hold a frequency for the DDS chip.
 *
 * The AD9850 uses a 32 bit (4 byte) frequency value.
 *
 * However, there is a signficant accumulation of round off
 * errors when only 4 bytes are used for intermediate calculations.
 * This is particularly noticable with the sweep generator.
 * So we basically add an extra fractional byte during
 * calculations and discard it when loading the AD9850.
 */

#define DDS_PREC 5         



/*--------------------------------------------------------------------
 *
 * Null command
 *
 * Does nothing.  Can be used for timing based on 30 characters/sec.
 *
 *--------------------------------------------------------------------*/

#define NULL_CMD 'N'


/*--------------------------------------------------------------------
 *
 * Sweep frequency command
 *
 *--------------------------------------------------------------------*/

#define SWEEP_CMD 'S'

struct sweep_cmd_s {
        byte_t lower_f[DDS_PREC];       /* Lowest frequency. */
        byte_t f_step[DDS_PREC];        /* Freq step for each X change. */
        byte_t n_marks;                 /* Number of calibration markers. */
        byte_t spacing;                 /* X steps between markers. */
};


/*--------------------------------------------------------------------
 *
 * Set frequency command
 *
 * Sets both the display and the DDS chip, just as if
 * was entered manually with the rotary encoder.
 *
 * Only the low order 4 bits of each character is used so
 * the numbers could be binary or ASCII digits.
 * Blank would be equivalent to a zero.
 *
 * Example:  For 123,456.7 Hz, command would be:
 *
 *              F001234567
 *
 *--------------------------------------------------------------------*/

#define FREQUENCY_CMD 'F'
                                        /* Followed by 'DIGITS' (9) */
                                        /* binary or ASCII numbers in */
                                        /* units of 0.1 Hz. */


/*--------------------------------------------------------------------
 *
 * Display message command
 *
 * Put characters on the display.
 * Does not affect the output frequency.
 *
 * The frequency can take up to 12 characters so only
 * the first 4 can be used without any danger of conflict.
 *
 * Cursor is put back where it was before this command.
 * Might have additional choices in the future.
 *
 *--------------------------------------------------------------------*/

#define MESSAGE_CMD 'M'

struct message_cmd_s {
        byte_t position;                /* Starting position: 0-15. */
        byte_t len;                     /* Number of characters: 1-16. */
        byte_t reserved;                /* For future use. */
};
                                        /* Followed by 'len' characters. */

/*--------------------------------------------------------------------
 *
 * Set Digitial to Analog Converter.
 *
 * There are 4 analog outputs:
 *
 *      A - Signal amplitude.  0=maximum, 255=minimum.
 *              Ideally the trim pot should be adjusted so the
 *              just goes to zero for maximum DAC output of
 *              about 1.25 volts.
 *
 *      B - Currently not used.
 *
 *      C - The "X" output (green binding post) used for the
 *              the sweep functions X axis.
 *
 *      D - The "Z" output (yellow binding post) used for the
 *              sweep function's calibration markers.
 *              This might be used to modulate the oscilloscope
 *              brightness or go to another Y channel.
 *
 * With 3 byte commands at 300 baud, we can update a DAC 10 times a second.
 *
 *--------------------------------------------------------------------*/

#define DAC_CMD 'A'

#define DAC_X DAC_C
#define DAC_Z DAC_D

struct dac_cmd_s {
        enum dac_addr_e { DAC_A=0, DAC_B=1, DAC_C=2, DAC_D=3} address;
        byte_t dac_data;                /* byte sent to the DAC. */
};



