/*----------------------------------------------------------------
 *
 * Name:        weight.c
 *
 * Purpose:     Calculate table of constants used for DDS signal
 *              generator based on the AD9850.
 *
 * Inputs:      Optional command line argument with oscillator
 *              frequency.
 *
 * Outputs:     Results of analysis to stdout.
 *              Table of constants to weight.h, ready for
 *              inclusion in the application.
 *
 * History:     Original, John Langner, WB2OSZ.
 *
 *----------------------------------------------------------------*/


#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <assert.h>

#include "sg_cmd.h"


void long5_from_double (byte_t b[DDS_PREC], double f)
{
        int n;

        f = floor(f + 0.5);
        for (n=0; n<DDS_PREC; n++) {

            b[n] = fmod(f, 256.);
            f = floor(f / 256.);
        }
}


void main (int argc, char *argv[]) {

        double xtal = DDS_CLOCK;        /* from sg_cmd.h */
        double cycle = pow (2.0, DDS_PREC * 8);
        double value[DIGITS];
        unsigned char weights[DIGITS][DDS_PREC];

        int digit, n;
        FILE *fp;


        assert (DDS_PREC >= 4);

/*
 * Optional command line argument for oscillator frequency.
 */

        if (argc > 1) {
            xtal = atof (argv[1]);
        }

        printf ("Using oscillator frequency of %lf MHz.\n\n",
                                xtal / 1000000.);

/*
 * Calculate weights for each digit position of display.
 */

        printf ("Digit     Value        Weight (dec, hex)\n");
        printf ("-----     -----        -----------------\n");

        for (digit=0; digit < DIGITS; digit++) {

            double w;

            value[digit] = pow10 (digit+8-DIGITS);
            w = value[digit] * cycle / xtal;

            /* The compiler I'm using doesn't have a 64 bit */
            /* "long long" integer so we'll do this in floating */
            /* point.  It's a whole number so we shouldn't have */
            /* any round off errors. */

            printf ("  %d %11.1lf %14.3lf  ", digit, value[digit],
                                w / pow(2.0, (DDS_PREC-4) * 8));

            long5_from_double (weights[digit], w);

            for (n=DDS_PREC-1; n>=0; n--) {

                printf (" %02x", weights[digit][n]);
                if (n == DDS_PREC-4)
                    printf (" .");
            }

            printf ("\n");
        }

/*
 * Sanity check and look at effects of round off.
 * Try running this with DDS_PREC of 4 and 5.
 *
 * The maximum error seen is < 0.07 Hz for 4 byte weights
 * and < 0.008 for 5 byte weights.
 * Increasing the precision to 6 digits doesn't have any
 * benefit because the chip has a resolution of about 1/64 Hz
 * with a 66 MHz clock.
 */

        printf ("\n");
        printf ("    Ideal            Actual       Error\n");
        printf ("    -----            ------       -----\n");

        for (digit=0; digit<DIGITS; digit++) {

            for (n=1; n<=9; n++) {

                double f, a;
                int j;

                if (digit == DIGITS-1 && n == 3)
                    break;

                f = n * value[digit];

                a = 0;
                for (j=DDS_PREC-1; j>=0; j--) {
                    a *= 256;
                    a += n * weights[digit][j];
                }

                /* Intermediate calculations may have had */
                /* more precision but we need to reduce to 32 */
                /* bits at this point to simulate sending it */
                /* to the DDS chip. */

                a = floor(a / pow(2.0, (DDS_PREC-4)*8) + 0.5);
                a *= xtal / pow(2.0, 32);

                printf ("%11.1lf %17.6lf %11.6lf\n", f, a, a-f);
            }

        }

/*
 * Finally create the .h file for inclusion in the application.
 */

        fp = fopen ("weight.h", "w");
        if (fp == NULL) {
            printf ("ERROR: can't create weight.h\n");
            exit (1);
        }


        fprintf (fp, "#asm\n");
        fprintf (fp, ";\n");   
        fprintf (fp, ";   weight.h - Created by weight.c program.\n");
        fprintf (fp, ";\n");   
        fprintf (fp, ";   %d digits, %d bytes each.\n", DIGITS, DDS_PREC);
        fprintf (fp, ";\n");   
        fprintf (fp, ";   NOTE: Must edit linker options and add\n");
        fprintf (fp, ";      -pdataee=2100h\n");
        fprintf (fp, ";\n");
        fprintf (fp, "  psect  dataee,pure,ovrld,delta=2\n");
        fprintf (fp, ";\n");


/*
 * Five digits of each weighting factor are adjacent.
 */
        for (digit=0; digit<DIGITS; digit++) {

            fprintf (fp, "  dw  ");

            for (n=0; n<DDS_PREC; n++) {
                fprintf (fp, "0x%02x%c", weights[digit][n], 
                        n < DDS_PREC-1 ? ',' : ' ');
            }
            fprintf (fp, "    ; %.1lf Hz\n", value[digit]);
        }

        fprintf (fp, ";\n");
        fprintf (fp, "#endasm\n");

        fclose (fp);
}
